﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using Infragistics.Silverlight;
using System.ComponentModel;
using System.Threading;
using System.Collections.ObjectModel;
using System.Windows.Data;
using System.Reflection;
using Infragistics.Silverlight.Controls;

namespace VC_View_Prj
{
    public partial class MainPage : UserControl
    {
        private VirtualCollection<TestData> _collection = null;// new VirtualCollection<TestData>();

        private List<TestData> _data;

        public MainPage()
        {
            InitializeComponent();

            this.Loaded += new RoutedEventHandler(MainPage_Loaded);
        }

        void MainPage_Loaded(object sender, RoutedEventArgs e)
        {
            _collection = new VirtualCollection<TestData>(25);
            _collection.PageSize = 10;
            _collection.ItemDataRequested += new EventHandler<ItemDataRequestedEventArgs>(_collection_ItemDataRequested);

            GenerateTestData(25);


            dataGrid1.ItemsSource = _collection;
            TestPager.Source = _collection;

        }

        void _collection_ItemDataRequested(object sender, ItemDataRequestedEventArgs e)
        {
            BackgroundWorker worker = new BackgroundWorker();

            worker.DoWork += new DoWorkEventHandler(worker_DoWork);
            worker.RunWorkerCompleted += new RunWorkerCompletedEventHandler(worker_RunWorkerCompleted);
            worker.RunWorkerAsync(e);

            //listBox1.Items.Add(string.Format("{0}:ItemDataRequested: StartIndex {1}, count: {2}", listBox1.Items.Count, e.StartIndex, e.ItemsCount));
            dataRequests.Items.Add(string.Format("{0}:ItemDataRequested: StartIndex {1}, count: {2}", dataRequests.Items.Count, e.StartIndex, e.ItemsCount));
        }

        #region Retrieve Data

        void GenerateTestData(int rowsCount)
        {
            _data = new List<TestData>();

            for (int i = 0; i < 101; i++)
            {
                _data.Add(new TestData()
                {
                    StringData = String.Format("Record num: {0}", i),
                    BoolData = ((i % 2) == 0),
                    IntegerData = i
                });
            }
        }

        void worker_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {

            ItemDataRequestedEventArgs args = e.Result as ItemDataRequestedEventArgs;

            var sortBy = new Dictionary<string, string>();
            foreach (var sdc in args.SortDescriptions)
            {
                string direction = string.Empty;
                if (sdc.Direction == ListSortDirection.Ascending)
                    direction = "ASC";
                if (sdc.Direction == ListSortDirection.Descending)
                    direction = "DESC";

                sortBy.Add(sdc.PropertyName, direction);
            }

            var dataToLoaded = GetData(args.StartIndex, args.StartIndex + args.ItemsCount, sortBy);

            this._collection.LoadItems(args.StartIndex, dataToLoaded);
        }

        void worker_DoWork(object sender, DoWorkEventArgs e)
        {
            Thread.Sleep(1000);
            e.Result = e.Argument;
        }
        public List<TestData> GetData(int startRow, int endRow, Dictionary<string, string> sortBy)
        {
            if (null == sortBy || 0 >= sortBy.Keys.Count)
            {
                return _data.Skip(startRow).Take(endRow - startRow).ToList<TestData>();
            }

            IEnumerable<TestData> source = _data;

            foreach (string propertyName in sortBy.Keys)
            {
                IOrderedEnumerable<TestData> orderedEnumerable = source as IOrderedEnumerable<TestData>;
                string propName = propertyName;
                if (null == orderedEnumerable)
                {
                    if ("ASC" == sortBy[propertyName])
                    {
                        source = source.OrderBy<TestData, object>(p => GetKeySelector(p, propName));
                    }
                    else
                    {
                        source = source.OrderByDescending<TestData, object>(p => GetKeySelector(p, propName));
                    }
                }
                else
                {
                    if ("ASC" == sortBy[propertyName])
                    {
                        source = orderedEnumerable.ThenBy<TestData, object>(p => GetKeySelector(p, propName));
                    }
                    else
                    {
                        source = orderedEnumerable.ThenByDescending<TestData, object>(p => GetKeySelector(p, propName));
                    }
                }
            }
            return source.Skip(startRow).Take(endRow - startRow).ToList<TestData>();

        }
        private object GetKeySelector(TestData p, string propertyName)
        {
            object target = p;
            target = target.GetType().InvokeMember(propertyName, BindingFlags.GetProperty, null, target, null);
            return target;
        }

        #endregion //Retrieve Data
    }
}
