﻿using System;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Windows.Media.Imaging;
using System.Collections;
using System.Collections.ObjectModel;
using System.Reflection;
using System.ComponentModel;
using System.Collections.Generic;

namespace VC_View_Prj
{
    public interface ITestData
    {
        string StringData { get; set; }
        int IntegerData { get; set; }
        decimal DecimalData { get; set; }
        double DoubleData { get; set; }
        bool BoolData { get; set; }
        bool? BoolDataNull { get; set; }

        void ConvertFrom(object data);
    }
    public class TestData : ITestData, INotifyPropertyChanged
    {
        #region ITestData Members

        public string StringData
        {
            get
            {
                return _stringData;
            }
            set
            {
                _stringData = value;
                OnPropertyChanged("StringData");
            }
        }

        public virtual int IntegerData
        {
            get
            {
                return _integerData;
            }
            set
            {
                _integerData = value;
                OnPropertyChanged("IntegerData");
            }
        }

        public decimal DecimalData
        {
            get
            {
                return _decimalData;
            }
            set
            {
                _decimalData = value;
                OnPropertyChanged("DecimalData");
            }
        }

        public double DoubleData
        {
            get
            {
                return _doubleData;
            }
            set
            {
                _doubleData = value;
                OnPropertyChanged("DoubleData");
            }
        }

        public bool BoolData
        {
            get
            {
                return _boolData;
            }
            set
            {
                _boolData = value;
                OnPropertyChanged("BoolData");
            }
        }

        public bool? BoolDataNull
        {
            get
            {
                return _boolDataNull;
            }
            set
            {
                _boolDataNull = value;
                OnPropertyChanged("BoolDataNull");
            }
        }

        public virtual void ConvertFrom(object data)
        {
            int index;
            List<PropertyInfo> dataPropertyInfos;

            if (data == null)
                return;

            dataPropertyInfos = new List<PropertyInfo>((data.GetType().GetProperties()));
            foreach (PropertyInfo thisPropertyInfo in this.GetType().GetProperties())
            {
                index = 0;
                for (int i = dataPropertyInfos.Count; index < i; index++)
                {
                    PropertyInfo dataPropertyInfo = dataPropertyInfos[index];

                    if (thisPropertyInfo.PropertyType == dataPropertyInfo.PropertyType)
                    {
                        thisPropertyInfo.SetValue(this, dataPropertyInfo.GetValue(data, null), null);
                        break;
                    }
                }

                if (index < dataPropertyInfos.Count)
                    dataPropertyInfos.RemoveAt(index);
            }
        }

        #endregion

        #region INotifyPropertyChanged Members

        public event PropertyChangedEventHandler PropertyChanged;

        #endregion

        #region Members

        private string _stringData;
        protected int _integerData;
        private decimal _decimalData;
        private double _doubleData;
        private bool _boolData;
        private bool? _boolDataNull;

        #endregion

        #region Constructors
        public TestData()
        {
        }

        public TestData(object data)
        {
            ConvertFrom(data);
        }

        #endregion

        #region Methods

        protected void OnPropertyChanged(string propertyName)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
        }

        #endregion
    }


}
